import { BaseElement } from '@emanon/shared/components/base';
import { customElement } from '@emanon/shared/decorators/custom-element';
import { html, property, PropertyValues } from 'lit-element';
import { style } from './index.gen.css';
import { style as darkStyle } from './dark.gen.css';
import { style as lightStyle } from './light.gen.css';
import prismjs from 'prismjs';

// @see https://github.com/PrismJS/prism/issues/1400#issuecomment-485847919
// phpを読み込むために必要
import 'prismjs/components/prism-markup-templating.min.js';

// 追加する言語
import 'prismjs/components/prism-bash.min.js';
import 'prismjs/components/prism-diff.min.js';
import 'prismjs/components/prism-docker.min.js';
import 'prismjs/components/prism-editorconfig.min.js';
import 'prismjs/components/prism-go.min.js';
import 'prismjs/components/prism-graphql.min.js';
import 'prismjs/components/prism-http.min.js';
import 'prismjs/components/prism-ignore.min.js';
import 'prismjs/components/prism-java.min.js';
import 'prismjs/components/prism-jq.min.js';
import 'prismjs/components/prism-json.min.js';
import 'prismjs/components/prism-jsx.min.js';
import 'prismjs/components/prism-kotlin.min.js';
import 'prismjs/components/prism-makefile.min.js';
import 'prismjs/components/prism-markdown.min.js';
import 'prismjs/components/prism-mongodb.min.js';
import 'prismjs/components/prism-nginx.min.js';
import 'prismjs/components/prism-perl.min.js';
import 'prismjs/components/prism-php.min.js';
import 'prismjs/components/prism-pug.min.js';
import 'prismjs/components/prism-python.min.js';
import 'prismjs/components/prism-ruby.min.js';
import 'prismjs/components/prism-rust.min';
import 'prismjs/components/prism-sql.min.js';
import 'prismjs/components/prism-swift.min.js';
import 'prismjs/components/prism-tsx.min.js';
import 'prismjs/components/prism-typescript.min.js';
import 'prismjs/components/prism-vim.min.js';
import 'prismjs/components/prism-wasm.min.js';
import 'prismjs/components/prism-yaml.min.js';

// @see https://prismjs.com/#supported-languages
const LANGUAGES = {
  '': 'Default',
  // prismjsのデフォルト言語
  markup: 'Markup(HTML, XML, SVG)',
  css: 'CSS',
  clike: 'C-like',
  js: 'JavaScript',
  // 追加する言語(AtoZ)
  bash: 'Bash',
  diff: 'Diff',
  docker: 'Docker',
  editorconfig: 'EditorConfig',
  go: 'Go',
  graphql: 'GraphQL',
  http: 'HTTP',
  ignore: '.ignore',
  java: 'Java',
  jq: 'JQ',
  json: 'JSON',
  jsx: 'React JSX',
  kotlin: 'Kotlin',
  makefile: 'Makefile',
  md: 'Markdown',
  mongodb: 'MongoDB',
  nginx: 'nginx',
  perl: 'Perl',
  php: 'PHP',
  pug: 'Pug',
  python: 'Python',
  ruby: 'Ruby',
  rust: 'Rust',
  sql: 'SQL',
  swift: 'Swift',
  tsx: 'React TSX',
  typescript: 'TypeScript',
  vim: 'vim',
  wasm: 'WebAssembly',
  yaml: 'YAML'
} as const;

/**
 * ## Overall
 * Highlighting ``<pre>`` & ``<code>`` block
 */
@customElement('ema-code-beautifier')
export class CodeBeautifier extends BaseElement {
  @property({ type: String })
  value = '';

  @property({ type: String, reflect: true })
  language: keyof typeof LANGUAGES = '';

  @property({ type: String, reflect: true })
  color: 'light' | 'dark' = 'dark';

  static get styles() {
    return [super.styles, style];
  }

  static get languages() {
    return LANGUAGES;
  }

  constructor() {
    super();
  }

  updated(changedProperties: PropertyValues) {
    super.updated(changedProperties);
    if (!this.shadowRoot) {
      return;
    }
    if (!changedProperties.has('language')) {
      return;
    }
    const elm = this.shadowRoot.querySelector('pre code');
    if (elm) {
      prismjs.highlightElement(elm);
    }
  }

  private get highlightTheme(): string {
    if (this.color === 'dark') {
      return darkStyle.cssText;
    }
    if (this.color === 'light') {
      return lightStyle.cssText;
    }
    return '';
  }

  render() {
    const { value, language, highlightTheme } = this;

    return html`
      <style>
        ${highlightTheme}
      </style>
      <pre><code class="language-${language}">${value}</code></pre>
    `;
  }
}
